/*
3D Mesh Editer via "A Simple Approach to Interactive Free-Form 
Shape Deformations" 
Copyright:(c) Shin Yoshizawa, 2002
E-mail: shin.yoshizawa@mpi-sb.mpg.de
URL: http://www.mpi-sb.mpg.de/~shin
Affiliation: Max-Planck-Institut fuer Informatik: Computer Graphics Group 
 Stuhlsatzenhausweg 85, 66123 Saarbruecken, Germany
 Phone +49 681 9325-414 Fax +49 681 9325-499 

 All right is reserved by Shin Yoshizawa.
This java program and sources are allowed for only primary user of 
research and educational purposes. Don't use secondary: copy, distribution, 
diversion, business purpose, and etc.. 

 */
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.loaders.Scene;
import com.sun.j3d.loaders.objectfile.ObjectFile;
import com.sun.j3d.loaders.ParsingErrorException;
import com.sun.j3d.loaders.IncorrectFormatException;
import com.sun.j3d.utils.behaviors.mouse.*;
import com.sun.j3d.utils.behaviors.interpolators.*;
import com.sun.j3d.utils.behaviors.picking.*;
import com.sun.j3d.utils.geometry.*;
    public class ObjectTransformEMesh {
      public TransformGroup objTrans;
      MouseRotate behavior;
      MouseZoom behavior2;
      MouseTranslate behavior3;
      Transform3D myscale=null;
      private Group root;
      private BranchGroup bg;
      private BranchGroup bgCP;
      TransparencyAttributes tattr;
      TransparencyAttributes cptattr;
      Shape3D myshape=null;
      Shape3D cpshape=null;
      Appearance app;
      Appearance cpapp;
      
      Material m;
      Material cpm;     
      PolygonAttributes pa;
      PolygonAttributes cppa;
      PointAttributes papa;
      Color3f eColor = new Color3f(0.0f, 0.0f, 0.0f);
      Color3f sColor = new Color3f(1.0f, 1.0f, 1.0f);
      Color3f normalc = new Color3f(0.2000f,0.6300f,0.7900f);
      
      
      public ObjectTransformEMesh(Shape3D shape, Group group){
	
	root = group;
	myshape = shape;

        objTrans = new TransformGroup();
        objTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	objTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
	
	objTrans.setCapability(TransformGroup.ENABLE_PICK_REPORTING);
	
	objTrans.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
	objTrans.setCapability(TransformGroup.ALLOW_CHILDREN_READ); 
	objTrans.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE); 
	
	 BoundingSphere bounds =
	   new BoundingSphere(new Point3d(0.0,0.0,0.0), 1.0);
    
	 
	 behavior = new MouseRotate();
	 behavior.setFactor((behavior.getXFactor()/6.0),
			    (behavior.getYFactor()/6.0));
	 behavior.setTransformGroup(objTrans);
	 objTrans.addChild(behavior);
	 behavior.setSchedulingBounds(bounds);
	 
	 
	 behavior2 = new MouseZoom();
	 behavior2.setFactor((behavior2.getFactor()/6.0));
	/* behavior2.setupCallback( new MouseBehaviorCallback() { 
	   public void transformChanged(int flag, Transform3D trans) {
	     myscale = new Transform3D(trans);
	     
	   }
	 });*/
	 behavior2.setTransformGroup(objTrans);
	 objTrans.addChild(behavior2);
	 behavior2.setSchedulingBounds(bounds);
	 
	 
	 behavior3 = new MouseTranslate();
	 behavior3.setFactor((behavior3.getXFactor()/6.0),
			    (behavior3.getYFactor()/6.0));

	 behavior3.setTransformGroup(objTrans);
	 objTrans.addChild(behavior3);
	 behavior3.setSchedulingBounds(bounds);
    

        bindShape(shape);
      }
      
      public void MouseOff(){
	behavior.setEnable(false);
	behavior2.setEnable(false);
	behavior3.setEnable(false);
	
      }
      public void MouseOn(){
	behavior.setEnable(true);
	behavior2.setEnable(true);
	behavior3.setEnable(true);

      }
      

      public void changeBackFace(boolean dv){
	
	  pa.setBackFaceNormalFlip(dv);
	  if(dv){
	    pa.setCullFace(pa.CULL_NONE);
	    
	  }else{
	    pa.setCullFace(pa.CULL_BACK);
	  }
      }
      public void changeMLight(boolean dv){
	if(dv){
	    m.setLightingEnable(dv);
	    app.setMaterial(m);
	  }else{
	    m.setLightingEnable(dv);
	    app.setMaterial(m);
	  }
      }
      public void changeBackFaceCP(boolean dv){
	
	  cppa.setBackFaceNormalFlip(dv);
	  if(dv){
	    cppa.setCullFace(cppa.CULL_NONE);
	    
	  }else{
	    cppa.setCullFace(cppa.CULL_BACK);
	  }
      }
      public void changeMLightCP(boolean dv){
	if(dv){
	    cpm.setLightingEnable(dv);
	    cpapp.setMaterial(cpm);
	  }else{
	    cpm.setLightingEnable(dv);
	    cpapp.setMaterial(cpm);
	  }
      }
      
      public void SetTransparency(float dv){
	tattr.setTransparency(dv);
	app.setTransparencyAttributes(tattr);
	myshape.setAppearance(app);
	
      }
      public void UnsetTransparency(){
	app.setTransparencyAttributes(null);
	myshape.setAppearance(app);
	
      }
      public void SetTransparencyCP(float dv){
	cptattr.setTransparency(dv);
	cpapp.setTransparencyAttributes(cptattr);
	cpshape.setAppearance(cpapp);
      }
      public void UnsetTransparencyCP(){
	cpapp.setTransparencyAttributes(null);
	cpshape.setAppearance(cpapp);
	
      }
      public void setRadius(float dv){
	papa.setPointSize(dv);
	app.setPointAttributes(papa);
      
      }
      
      public void bindShape(Shape3D shape) {
	app = new Appearance();
	app.setCapability(app.ALLOW_POINT_ATTRIBUTES_WRITE);
	app.setCapability(app.ALLOW_POLYGON_ATTRIBUTES_WRITE);
	app.setCapability(app.ALLOW_MATERIAL_WRITE);
	app.setCapability(app.ALLOW_TRANSPARENCY_ATTRIBUTES_WRITE);
	
	
	
	tattr = new TransparencyAttributes(TransparencyAttributes.BLENDED,0.6f);
	tattr.setCapability(TransparencyAttributes.ALLOW_VALUE_READ); 
	tattr.setCapability(TransparencyAttributes.ALLOW_VALUE_WRITE);
	tattr.setTransparencyMode(TransparencyAttributes.BLENDED);
	
	app.setTransparencyAttributes(tattr);
	app.setTransparencyAttributes(null);

	pa = new PolygonAttributes();
	pa.setCapability(pa.ALLOW_MODE_WRITE);
	pa.setCapability(pa.ALLOW_NORMAL_FLIP_WRITE);
	pa.setCapability(pa.ALLOW_CULL_FACE_WRITE);
	
	
	pa.setBackFaceNormalFlip(true);
	pa.setCullFace(pa.CULL_NONE);
	app.setPolygonAttributes(pa);
			
	papa = new PointAttributes();
	papa.setPointSize(2.5f);
	app.setPointAttributes(papa);

	m = new Material(normalc, eColor, normalc, sColor, 100.0f);
	m.setCapability(m.ALLOW_COMPONENT_WRITE);
	m.setLightingEnable(true);
	app.setMaterial(m);
	
	shape.setAppearance(app);
	shape.setCapability(shape.ALLOW_APPEARANCE_WRITE);
	shape.setCapability(shape.ALLOW_APPEARANCE_READ);
	
	shape.setCapability(shape.ALLOW_GEOMETRY_WRITE);
	shape.setCapability(shape.ALLOW_GEOMETRY_READ);
	
	objTrans.addChild(shape);

	bg = new BranchGroup();
	bg.setCapability(BranchGroup.ALLOW_DETACH);
	
	bg.addChild(objTrans);
	bg.compile(); 
	root.addChild(bg);
      }
       public void addCP(Shape3D shape) {
	 cpshape = shape;
	 cpapp = new Appearance();
	 cpapp.setCapability(app.ALLOW_POINT_ATTRIBUTES_WRITE);
	 cpapp.setCapability(app.ALLOW_POLYGON_ATTRIBUTES_WRITE);
	 cpapp.setCapability(app.ALLOW_MATERIAL_WRITE);
	 cpapp.setCapability(app.ALLOW_TRANSPARENCY_ATTRIBUTES_WRITE);
	

	cptattr = new TransparencyAttributes(TransparencyAttributes.BLENDED,0.5f);
	cptattr.setCapability(TransparencyAttributes.ALLOW_VALUE_READ); 
	cptattr.setCapability(TransparencyAttributes.ALLOW_VALUE_WRITE);
	cptattr.setTransparencyMode(TransparencyAttributes.BLENDED);
	
	cpapp.setTransparencyAttributes(cptattr);
	cpapp.setTransparencyAttributes(null);

	cppa = new PolygonAttributes();
	cppa.setCapability(cppa.ALLOW_MODE_WRITE);
	cppa.setCapability(cppa.ALLOW_NORMAL_FLIP_WRITE);
	cppa.setCapability(cppa.ALLOW_CULL_FACE_WRITE);
	
	
	cppa.setBackFaceNormalFlip(true);
	cppa.setCullFace(cppa.CULL_NONE);
	cpapp.setPolygonAttributes(cppa);
	Color3f cpcolor = new Color3f(0.0f,1.000f,0.0f);
	cpm = new Material(cpcolor, eColor, cpcolor, sColor, 100.0f);
	cpm.setCapability(cpm.ALLOW_COMPONENT_WRITE);
	cpm.setLightingEnable(true);
	cpapp.setMaterial(cpm);
	
	cpshape.setAppearance(cpapp);
	cpshape.setCapability(shape.ALLOW_APPEARANCE_WRITE);
	cpshape.setCapability(shape.ALLOW_APPEARANCE_READ);
	
	cpshape.setCapability(shape.ALLOW_GEOMETRY_WRITE);
	cpshape.setCapability(shape.ALLOW_GEOMETRY_READ);
	
	bgCP = new BranchGroup();
	bgCP.setCapability(BranchGroup.ALLOW_DETACH);
	bgCP.addChild(cpshape);
	objTrans.addChild(bgCP);
      }
       

      

      
      public void bindCP() {
        
	objTrans.addChild(bgCP);
      }
      public void unbindCP() {
        bgCP.detach();
      }
      
      public void bind() {
	
	 root.addChild(bg);
	
       }
      public void unbind() {
        
	bg.detach();
      }
      
      public void Surface(){
	
	pa.setPolygonMode(PolygonAttributes.POLYGON_FILL);
	app.setPolygonAttributes(pa);
	app.setMaterial(m);
	myshape.setAppearance(app);
      }
      public void Point(){
	
	pa.setPolygonMode(PolygonAttributes.POLYGON_POINT);
	app.setPolygonAttributes(pa);
	app.setMaterial(m);
	myshape.setAppearance(app);
      }
      public void WireFrame(){
	
	pa.setPolygonMode(PolygonAttributes.POLYGON_LINE);
	app.setPolygonAttributes(pa);
	app.setMaterial(m);
	myshape.setAppearance(app);
      }
      public void SurfaceCP(){
	
	cppa.setPolygonMode(PolygonAttributes.POLYGON_FILL);
	cpapp.setPolygonAttributes(cppa);
	cpapp.setMaterial(cpm);
	cpshape.setAppearance(cpapp);
      }
      public void PointCP(){
	
	cppa.setPolygonMode(PolygonAttributes.POLYGON_POINT);
	cpapp.setPolygonAttributes(cppa);
	cpapp.setMaterial(cpm);
	cpshape.setAppearance(cpapp);
      }
      public void WireFrameCP(){
	
	cppa.setPolygonMode(PolygonAttributes.POLYGON_LINE);
	cpapp.setPolygonAttributes(cppa);
	cpapp.setMaterial(cpm);
	cpshape.setAppearance(cpapp);
      }
      public  BranchGroup getRoot(){
	return bgCP;
      }
    }
