/*
3D Mesh Editer via "A Simple Approach to Interactive Free-Form 
Shape Deformations" 
Copyright:(c) Shin Yoshizawa, 2002
E-mail: shin.yoshizawa@mpi-sb.mpg.de
URL: http://www.mpi-sb.mpg.de/~shin
Affiliation: Max-Planck-Institut fuer Informatik: Computer Graphics Group 
 Stuhlsatzenhausweg 85, 66123 Saarbruecken, Germany
 Phone +49 681 9325-414 Fax +49 681 9325-499 

 All right is reserved by Shin Yoshizawa.
This java program and sources are allowed for only primary user of 
research and educational purposes. Don't use secondary: copy, distribution, 
diversion, business purpose, and etc.. 

 */
import java.awt.*;
import java.applet.*;
import java.util.*;
import java.awt.event.*;
import java.io.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class Polyhedron{
  
  ViewShape3D myobj=null;
  
  
  boolean blend = false;
  double aveH;
  double sdviH;
  double threshold = 1.5;
  double scaler =1.0;
  double beta = 2.0;
  
  double[] H=null;
  ControlInterface mycontrol=null;
  
  
  
  Point3d[] path=null;
  
  Point3d[] KnotInit=null;
  int numberV=0;
  int numberF=0;
  int numberE=0;
  int EulerC=0;
  Point3d[] point=null;
  int[][] Face;
  int[] neighborI=null;
  int[] neighborF=null;
  int[] neighborV=null;
  IDList[] FHead=null;
  IDList[] FTail=null;
  IDList[] VHead=null;
  IDList[] VTail=null;
  IDList[] IHead=null;
  IDList[] ITail=null;
  int[] boundary=null;
  
  
  int numboundary=0;
  
  Vector3d ddnormalv3d = new Vector3d(0.0,0.0,0.0);
  Vector3d v1v3d = new Vector3d(0.0,0.0,0.0);
  Vector3d v2v3d = new Vector3d(0.0,0.0,0.0);
  Point3d tmppoint3d1 = new Point3d(0.0,0.0,0.0);
  Point3d tmppoint3d2 = new Point3d(0.0,0.0,0.0);
  
  
  public Polyhedron(){
  }
  private void MemoryAllocation(){
  
    
    
    
    path = new Point3d[numberV];
    point = new Point3d[numberV];
    H = new double[numberV];
    //oH = new double[numberV];
    boundary = new int[numberV];
    neighborV = new int[numberV];
    neighborF  = new int[numberV];
    neighborI = new int[numberV];
    VHead = new IDList[numberV];
    VTail = new IDList[numberV];
    FHead = new IDList[numberV];
    FTail = new IDList[numberV];
    IHead = new IDList[numberV];
    ITail = new IDList[numberV];
    KnotInit = new Point3d[numberV];
    Face = new int[numberF][3];
  }
  private void InitializeData(int di,double dx,double dy,double dz){
     point[di] = new Point3d(dx,dy,dz);
     path[di] = new Point3d(0.0,0.0,0.0);
         
     H[di] = 0.0;
     //oH[di] = 0.0;
     
     neighborV[di]=0;
     neighborF[di]=0;
     neighborI[di]=0;
     VHead[di] = new IDList();
     VTail[di]  = new IDList();
     this.Init(VHead[di],VTail[di]);
     FHead[di] = new IDList();
     FTail[di]  = new IDList();
     this.Init(FHead[di],FTail[di]);
     IHead[di] = new IDList();
     ITail[di]  = new IDList();
     this.Init(IHead[di],ITail[di]);
     KnotInit[di] = new Point3d(dx,dy,dz);	
    
  }
  public void MakeCP(ControlInterface dcp){
     mycontrol = dcp;
  }
  private void MakeFace(int i,int di,int dj,int dk){
    Face[i][0] = di;
    Face[i][1] = dj;
    Face[i][2] = dk;
    
    /* One */
    this.AppendVF(i,FTail[di]);neighborF[di]++;
    this.AppendI(dj,IHead[di],ITail[di],di,neighborI);
    this.AppendI(dk,IHead[di],ITail[di],di,neighborI);
    this.AppendVF(dj,VTail[di]);neighborV[di]++;
    this.AppendVF(dk,VTail[di]);neighborV[di]++;
    /* Two */
    this.AppendVF(i,FTail[dj]);neighborF[dj]++;
    this.AppendI(di,IHead[dj],ITail[dj],dj,neighborI);
    this.AppendI(dk,IHead[dj],ITail[dj],dj,neighborI);
    this.AppendVF(dk,VTail[dj]);neighborV[dj]++;
    this.AppendVF(di,VTail[dj]);neighborV[dj]++;
    /* Three */
    this.AppendVF(i,FTail[dk]);neighborF[dk]++;
    this.AppendI(di,IHead[dk],ITail[dk],dk,neighborI);
    this.AppendI(dj,IHead[dk],ITail[dk],dk,neighborI);
    this.AppendVF(di,VTail[dk]);neighborV[dk]++;
    this.AppendVF(dj,VTail[dk]);neighborV[dk]++;
    
  }
  public void Init(IDList h,IDList t)
    {
      h.next = new IDList();
      t.back = new IDList();
      h.next = t;
      t.back = h;
    }
  private ControlPoint next(ControlPoint now){
      return (now.next);
    }
   private ControlPoint back(ControlPoint now){
      return (now.back);
    }
    
   public IDList next(IDList now)
    {
      return (now.next);
    }
  public IDList back(IDList now)
    {
      return (now.back);
    }
  
  public void AppendVF(int myID,IDList dVTail)
    {
      IDList now = new IDList(myID);
      
      IDList dummy = dVTail.back;
      now.next = dVTail;
      dVTail.back = now;
      now.back = dummy;
      dummy.next =now;
    } 
  
  private boolean SearchI(int dID,IDList dIHead,IDList dITail){
    
    IDList now = dIHead;
    while(next(now)!=dITail){
      now = next(now);
      if((now.ID==dID)){
	return true;
      }
    }
    return false;
  }
 
  
  public void AppendI(int dID,IDList dIHead,IDList dITail,int nowID,int[] dnum)
    {
      if(dID!=nowID){
	if(!SearchI(dID,dIHead,dITail)){
	  IDList now = new IDList(dID);
	  IDList dummy = dITail.back;
	  now.next = dITail;
	  dITail.back = now;
	  now.back = dummy;
	  dummy.next =now;
	  dnum[nowID]++;
	}
      }else{
	//System.out.println("Check !!!");
      }
    }
 
  public boolean ReadTPolyData(String filename){
    
    try{
      FileInputStream  input = 
	new FileInputStream (filename);
      
      LineNumberReader lnr = 
	new LineNumberReader(new BufferedReader(new InputStreamReader(input)));
      
      StringTokenizer st;
      
      
      int dnumV,dnumF;
      
      st = new StringTokenizer(lnr.readLine());
      dnumV = (Integer.valueOf(st.nextToken())).intValue();
      if(!st.hasMoreTokens())
	st = new StringTokenizer(lnr.readLine());
      dnumF = (Integer.valueOf(st.nextToken())).intValue();
      
      numberF = dnumF;
      numberV = dnumV;
      numberF = dnumF;
      MemoryAllocation();
      
      int i,j,k;
      
      //System.out.println("Init finish.");
      double dx,dy,dz;
       double maxvalue = 0.0;
      
      for(i=0;i<numberV;i++){
      if(!st.hasMoreTokens())
	st = new StringTokenizer(lnr.readLine());
      dx = (Double.valueOf(st.nextToken())).doubleValue();
      if(!st.hasMoreTokens())
	st = new StringTokenizer(lnr.readLine());
      dy = (Double.valueOf(st.nextToken())).doubleValue();
      if(!st.hasMoreTokens())
	st = new StringTokenizer(lnr.readLine());
      dz = (Double.valueOf(st.nextToken())).doubleValue();
      InitializeData(i,dx,dy,dz);
      if(maxvalue<Math.abs(dx))maxvalue = Math.abs(dx);
	if(maxvalue<Math.abs(dy))maxvalue = Math.abs(dy);
	if(maxvalue<Math.abs(dz))maxvalue = Math.abs(dz);
      
      }
      
      
      if(maxvalue != 0.0){
	scaler = 1.0/maxvalue;
      System.out.println("scaler = "+scaler);
    for(i=0;i<numberV;i++){
      point[i].x *= scaler;
      point[i].y *= scaler;
      point[i].z *= scaler;
      KnotInit[i].x *= scaler;
      KnotInit[i].y *= scaler;
      KnotInit[i].z *= scaler;
    }
      }
    
    int di,dj,dk;
    int dummm;
      for(i=0;i<numberF;i++){
	if(!st.hasMoreTokens())
	  st = new StringTokenizer(lnr.readLine());
	dummm = (Integer.valueOf(st.nextToken())).intValue();
	if(!st.hasMoreTokens())
	  st = new StringTokenizer(lnr.readLine());
	di = (Integer.valueOf(st.nextToken())).intValue();
	if(!st.hasMoreTokens())
	  st = new StringTokenizer(lnr.readLine());
	dj = (Integer.valueOf(st.nextToken())).intValue();
	if(!st.hasMoreTokens())
	  st = new StringTokenizer(lnr.readLine());
	dk = (Integer.valueOf(st.nextToken())).intValue();
	
	MakeFace(i,di,dj,dk);
      }
      
      this.SetBoundaryStrips();
      
      numberE = 0;
      for(i=0;i<numberV;i++){
	numberE += (neighborI[i]);
      }
      numberE /= 2;
      EulerC = numberV - numberE + numberF;
      System.out.println("V = "+numberV+", F = "+numberF+", E = "+numberE);
      System.out.println("EulerC = "+EulerC);
      
      myobj = new ViewShape3D(3);      
      MakeArrayData();
      PropertyUpdate();
      input.close();
      return true;
     }catch (IOException e){
      System.out.println("IOException");
      return false;
    }
  }
  public void SetBoundaryLines(){
    int i=0;numboundary=0;
    if(neighborI!=null && neighborF!=null && boundary!=null)
      for(i=0;i<numberV;i++){
	if(((neighborI[i]) == neighborF[i]) && 
	   (neighborF[i] !=0) && 
	   (neighborI[i] !=0)){
	  boundary[i] = 0;
	}else{
	  boundary[i] = 1;
	  numboundary++;
	}
      }
    //System.out.println("number of boundary points = "+numboundary);
  
  }
  public void SetBoundaryStrips(){
    int i=0;
    IDList now;
     int[] dummyboundary=null;numboundary=0;
    if(neighborI!=null && neighborF!=null && boundary!=null){
      dummyboundary = new int[numberV];
      for(i=0;i<numberV;i++){
	if(((neighborI[i]) == neighborF[i]) && 
	   (neighborF[i] !=0) && 
	   (neighborI[i] !=0)){
	  dummyboundary[i] = 0;
	}else{
	  dummyboundary[i] = 1;
	}
	boundary[i] = 0;
      }
      
      for(i=0;i<numberV;i++){
	if(dummyboundary[i]==1){
	  boundary[i] = 1;
	  //deeplevel[i] = 1;
	  numboundary++;
	  now = VHead[i];
	  while(next(now)!=VTail[i]){
	    now = next(now);
	    if(boundary[now.ID]!=1)
	      boundary[now.ID] = 2;
	    now = next(now);
	  }
	}
      }
    }
    
    System.out.println("number of boundary points = "+numboundary);   
   
  }
  public void makeVector(Point3d out,Point3d in1,Point3d in2){
    out.x = (in2.x - in1.x);
    out.y = (in2.y - in1.y);
    out.z = (in2.z - in1.z);
  }
  public void makeVector(Point3d out,Vertex in1,Vertex in2){
    out.x = ((in2.p).x - (in1.p).x);
    out.y = ((in2.p).y - (in1.p).y);
    out.z = ((in2.p).z - (in1.p).z);
  }
  public void CrossVector(Point3d out,Point3d in1,Point3d in2){
  out.x = (in1.y*in2.z - in2.y*in1.z);
  out.y = (in1.z*in2.x - in2.z*in1.x);
  out.z = (in1.x*in2.y - in2.x*in1.y);
  }
  public double InnerProduct(Point3d in1,Point3d in2){
    return (in1.x*in2.x + in1.y*in2.y + in1.z*in2.z);
  }
  public void ScalarVector(Point3d out,double dv,Point3d in){
    out.x = (dv*in.x);
    out.y = (dv*in.y);
    out.z = (dv*in.z);
  }
  
  public double Point3dSize(Point3d in){
    return Math.sqrt(((in.x*in.x)+(in.y*in.y)+(in.z*in.z)));
  }
  public double Distance(Point3d in1,Point3d in2){
    return Math.sqrt((((in1.x-in2.x)*(in1.x-in2.x))
		      +((in1.y-in2.y)*(in1.y-in2.y))
		      +((in1.z-in2.z)*(in1.z-in2.z))));
		      
  }
  public double EuclidDistance(Point3d in1,Point3d in2){
    return Math.sqrt((((in1.x-in2.x)*(in1.x-in2.x))
		      +((in1.y-in2.y)*(in1.y-in2.y))
		      +((in1.z-in2.z)*(in1.z-in2.z))));
		      
  }
  
  
  private void PropertyUpdate(){
    int i,j;
    Point3d[] dv = new Point3d[9];
    IDList now;
    boolean secondfirst=true;
    for(i=0;i<9;i++)dv[i] = new Point3d(0.0,0.0,0.0);
    double  tarea = 0.0;
    double area = 0.0;
    aveH=0.0;
    sdviH = 0.0;   
    double divisor=0.0;
    double dummytemp=0.0;
    double InnW1=0.0;
    double InnW2=0.0;
    
    for(i=0;i<numberV;i++){
      if(boundary[i] !=1){
	divisor += 1.0;
	now = VHead[i];
	area = 0.0;
	tarea = 0.0;
	for(j=0;j<9;j++){
	  dv[j].x = 0.0;dv[j].y = 0.0;dv[j].z = 0.0;
	}
	
	while(next(now)!=VTail[i]){
	  now = next(now);
	  
	  this.makeVector(dv[0],point[i],point[next(now).ID]);
	  this.makeVector(dv[1],point[next(now).ID],point[now.ID]);
	  this.makeVector(dv[2],point[i],point[now.ID]);
	  this.makeVector(dv[3],point[now.ID],point[next(now).ID]);
	  this.CrossVector(dv[4],dv[2],dv[0]);
	  area = ((1.0/2.0)*(Point3dSize(dv[4])));
	  InnW1 = InnerProduct(dv[0],dv[1]);
	  InnW2 = InnerProduct(dv[2],dv[3]);
	  tarea += area;
	  this.ScalarVector(dv[5],InnW1,dv[2]);
	  this.ScalarVector(dv[6],InnW2,dv[0]);
	  if(area != 0.0){
	    dv[7].x += ((1.0/(4.0*area))*(dv[5].x + dv[6].x));
	    dv[7].y += ((1.0/(4.0*area))*(dv[5].y + dv[6].y));
	    dv[7].z += ((1.0/(4.0*area))*(dv[5].z + dv[6].z));
	    dv[8].x += dv[4].x;
	    dv[8].y += dv[4].y;
	    dv[8].z += dv[4].z;
	    
	  }
	  now = next(now);
	}
	dummytemp = Point3dSize(dv[8]);
	if(dummytemp != 0.0){
	  tmppoint3d1.x = ((dv[8].x)/dummytemp);
	  tmppoint3d1.y = ((dv[8].y)/dummytemp);
	  tmppoint3d1.z = ((dv[8].z)/dummytemp);
	}
	
	if(tarea != 0.0){
	  tmppoint3d2.x = -((dv[7].x)/(2.0*tarea));
	  tmppoint3d2.y = -((dv[7].y)/(2.0*tarea));
	  tmppoint3d2.z = -((dv[7].z)/(2.0*tarea));
	}
	H[i] = InnerProduct(tmppoint3d1,tmppoint3d2);
      }else{
	H[i] = 0.0;
      }
	
      if(secondfirst){
	secondfirst=false;
	
	aveH += H[i];
	  sdviH += (H[i]*H[i]);
      }else{
	
	aveH += H[i];
	sdviH += (H[i]*H[i]);
      }
    }
    
         
     
    aveH /= divisor;
    sdviH -= (aveH*aveH);
    sdviH /= divisor;
    sdviH = Math.sqrt(sdviH);
    setColorPrH();
  }
  public void setColorPrH(){
    
    int i;
    double dumMax = 0.0;
    double dumMin=0.0;
    double[] k = new double[numberV];
    
    double thr = threshold*sdviH;
    for(i=0;i<numberV;i++){
      if(boundary[i] != 1){
	if(Math.abs((H[i] - aveH)) >= thr){
	  if((H[i] - aveH)<0.0){
	    k[i] = aveH - thr;
	  }else{
	    k[i] = aveH + thr;
	  }
	}else{
	  k[i] = H[i];
	}
      }
    }
    boolean first=true;
    for(i=0;i<numberV;i++){
      if(boundary[i] != 1){
	if(first){
	  first=false;
	  dumMax = k[i];
	  dumMin = dumMax;
	}else{
	  if(k[i]>dumMax)dumMax = k[i];
	  if(k[i]<dumMin)dumMin = k[i];
	}
      }
    }
    
    for(i=0;i<numberV;i++)
      if(boundary[i] != 1){
	k[i] -= dumMin;
	k[i] /= (dumMax-dumMin);
	if(k[i]<0)k[i] = 0.0;
	if(k[i]>1.0)k[i] = 1.0;
      }
    myobj.setPastelColor(k,Face);
    
    
  } 
    
 
  private void MakeArrayData(){
    int i=0;
    int j=0;
    
   
    myobj.reNew(numberV,numberF);
    
    int k=0;
    
    for(j=0;j<numberF;j++)
      myobj.MakeFace(j,Face[j][0],Face[j][1],Face[j][2],point);
    
    
    myobj.MakeArrayData();
  }
  public void InitPosition(){
    int i,j;
    for(i=0;i<numberV;i++){
      
      point[i].x = KnotInit[i].x;
      point[i].y = KnotInit[i].y;
      point[i].z = KnotInit[i].z;
    }
    PropertyUpdate();
    UpdatePolygon();
    

  }
  
  
  public void UpdateCoordinate(int numenettimes){
    int i;
   
    
    for(i=0;i<numenettimes;i++){
      PropertyUpdate();
      setNetwork();
      DMeshAlgorithm();
      System.out.print(".");
    }
    UpdatePolygon();
      
  }
  public void UpdateCoordinate(){
    PropertyUpdate();
    setNetwork();
    DMeshAlgorithm();UpdatePolygon();
    System.out.print(".");
  }
  public void setMinID(){
    int j=0;
    
    ControlPoint now;
    double length = 0.0;
    double minlength=0;
    int minID=0;
        
    now = mycontrol.pHead;
    while(next(now)!=mycontrol.pTail){
      now = next(now);
      length = EuclidDistance(now.point,KnotInit[0]);
      minlength = length;
      minID=0;
      for(j=0;j<numberV;j++){
	length = EuclidDistance(now.point,KnotInit[j]);
	if(minlength>length){
	  minlength = length;
	  minID=j;
	}
      }
      now.minID = minID;
     /* if(now.cpstate==0||now.cpstate==1){
	now.InitAnisoMetric(KnotInit[minID]);
      }else{
	now.InitAnisoMetric(now.Ipoint);
      }
      */
      now.InitAnisoMetric(KnotInit[minID]);
     
    }
      
  }
  private void setNetwork(){
    int j=0;
    
    ControlPoint now;
    double length = 0.0;
    double weight = 0.0;
    double maxweight=0.0;
    double ddx=0.0;
    double ddy=0.0;
    double ddz=0.0;
    j=0;
    
    setMinID();
    now = mycontrol.pHead;
    while(next(now)!=mycontrol.pTail){
      now = next(now);
      maxweight=0.0;
      if(now.cpstate==0||now.cpstate==2){
	now.Vpoint.x = now.point.x;
	now.Vpoint.y = now.point.y;
	now.Vpoint.z = now.point.z;
      }else if(now.cpstate==1){
	now.Vpoint.x = 2.0*KnotInit[now.minID].x - now.point.x;
	now.Vpoint.y = 2.0*KnotInit[now.minID].y - now.point.y;
	now.Vpoint.z = 2.0*KnotInit[now.minID].z - now.point.z;
      }else if(now.cpstate==3){
	now.Vpoint.x = 2.0*now.Ipoint.x - now.point.x;
	now.Vpoint.y = 2.0*now.Ipoint.y - now.point.y;
	now.Vpoint.z = 2.0*now.Ipoint.z - now.point.z;
      }
      if(now.cpstate==0||now.cpstate==1){
	if(now.w){
	  length = now.AM.AnisotropicDistance(now.point,KnotInit[0]);
	}else{
	  length = now.AM.AnisotropicDistance(KnotInit[now.minID],KnotInit[0]);
	}
      }else{
	if(now.w){
	  length = now.AM.AnisotropicDistance(now.point,KnotInit[0]);
	}else{
	  length = now.AM.AnisotropicDistance(now.Ipoint,KnotInit[0]);
	}
      }
      weight = Math.exp((-(Math.pow(length,now.alpha))/(2.0*now.epsilon*now.epsilon)));
      maxweight = weight;
      
      now.Eweight[0] = length;
      
      for(j=0;j<numberV;j++){
	if(now.cpstate==0||now.cpstate==1){
	  if(now.w){
	    length = now.AM.AnisotropicDistance(now.point,KnotInit[j]);
	  }else{
	    length = now.AM.AnisotropicDistance(KnotInit[now.minID],KnotInit[j]);
	  }
	}else{
	  if(now.w){
	    length = now.AM.AnisotropicDistance(now.point,KnotInit[j]);
	  }else{
	    length = now.AM.AnisotropicDistance(now.Ipoint,KnotInit[j]);
	  }
	}
	weight = Math.exp((-(Math.pow(length,now.alpha))/(2.0*now.epsilon*now.epsilon)));
	
	if(maxweight<weight){
	  maxweight = weight;
	}
	now.Eweight[j] = weight;
	
      }
       if(now.w){
	  
	    length = now.AM.AnisotropicDistance(now.point,KnotInit[now.minID]);
	    maxweight = Math.exp((-(Math.pow(length,now.alpha))/(2.0*now.epsilon*now.epsilon)));
       }
     
      if(maxweight==0.0){
	maxweight=1.0;
      }
      if(now.w){
	now.norm = maxweight;
      }else{
	now.norm = 1.0;
      }
      
    }
    
    double ddxx=0.0;
    double ddyy=0.0;
    double ddzz=0.0;
    double dist=0.0;
    double sumdis=0.0;
    
    if(blend){
     
      for(j=0;j<numberV;j++){
	
	sumdis=0.0;
	path[j].x = 0.0;
	path[j].y = 0.0;
	path[j].z = 0.0;
	now = mycontrol.pHead;
	while(next(now)!=mycontrol.pTail){
	  now = next(now);
	  ddxx = now.gamma*(now.Eweight[j]*(now.Vpoint.x - KnotInit[j].x)/now.norm);
	  ddyy = now.gamma*(now.Eweight[j]*(now.Vpoint.y - KnotInit[j].y)/now.norm);
	  ddzz = now.gamma*(now.Eweight[j]*(now.Vpoint.z - KnotInit[j].z)/now.norm);
	  
	  
	  dist = Math.pow(Math.sqrt((ddxx*ddxx+ddyy*ddyy+ddzz*ddzz)),beta);
	  
	  sumdis += dist;
	  
	  path[j].x -= dist*ddxx;
	  path[j].y -= dist*ddyy;
	  path[j].z -= dist*ddzz;
	
	}
	if(sumdis!=0.0){
	   path[j].x /= sumdis;
	   path[j].y /= sumdis;
	   path[j].z /= sumdis;
	 }
      }
      
    }else{
       for(j=0;j<numberV;j++){
	
	sumdis=0.0;
	path[j].x = 0.0;
	path[j].y = 0.0;
	path[j].z = 0.0;
	now = mycontrol.pHead;
	while(next(now)!=mycontrol.pTail){
	  now = next(now);
	  
	  ddxx = now.gamma*(now.Eweight[j]*(now.Vpoint.x - KnotInit[j].x)/now.norm);
	  ddyy = now.gamma*(now.Eweight[j]*(now.Vpoint.y - KnotInit[j].y)/now.norm);
	  ddzz = now.gamma*(now.Eweight[j]*(now.Vpoint.z - KnotInit[j].z)/now.norm);
	  
	  path[j].x -= ddxx;
	  path[j].y -= ddyy;
	  path[j].z -= ddzz;
	
	}
	
	
      }
    
    }

  }
  private void setNetworkABC(){
    int j=0;
    int i=0;
    ControlPoint now;
    double length = 0.0;
    double weight = 0.0;
    
    double maxweight=0.0;
    double minlength=0;
    int minID=0;
    
    double ddx=0.0;
    double ddy=0.0;
    double ddz=0.0;
    j=0;
    i=0;
    now = mycontrol.pHead;
    while(next(now)!=mycontrol.pTail){
      now = next(now);
      maxweight=0.0;
      minlength=0.0;
      minID=0;
      
      if(now.cpstate==0||now.cpstate==1){
	length = EuclidDistance(now.point,KnotInit[0]);
      }else{
	length = EuclidDistance(now.Ipoint,KnotInit[0]);
      }
      weight = Math.exp((-(Math.pow(length,now.alpha))/(2.0*now.epsilon*now.epsilon)));
      maxweight = weight;
      minlength = length;
      minID=0;
      now.Eweight[0] = length;
      
      for(j=0;j<numberV;j++){
	if(now.cpstate==0||now.cpstate==1){
	  length = EuclidDistance(now.point,KnotInit[j]);
	}else{
	  length = EuclidDistance(now.Ipoint,KnotInit[j]);
	}
	weight = Math.exp((-(Math.pow(length,now.alpha))/(2.0*now.epsilon*now.epsilon)));
	
	if(maxweight<weight){
	  maxweight = weight;
	}
	if(minlength>length){
	  minlength = length;
	  minID=j;
	}
	now.Eweight[j] = weight;
	
      }
      if(now.cpstate==0||now.cpstate==2){
	now.Vpoint.x = now.point.x;
	now.Vpoint.y = now.point.y;
	now.Vpoint.z = now.point.z;
      }else if(now.cpstate==1){
	now.Vpoint.x = 2.0*KnotInit[minID].x - now.point.x;
	now.Vpoint.y = 2.0*KnotInit[minID].y - now.point.y;
	now.Vpoint.z = 2.0*KnotInit[minID].z - now.point.z;
      }else if(now.cpstate==3){
	now.Vpoint.x = 2.0*now.Ipoint.x - now.point.x;
	now.Vpoint.y = 2.0*now.Ipoint.y - now.point.y;
	now.Vpoint.z = 2.0*now.Ipoint.z - now.point.z;
      }
      
      if(maxweight==0.0){
	maxweight=1.0;
      }
      if(now.cpstate==0||now.cpstate==1){
	now.norm = maxweight;
      }else{
	now.norm = 1.0;
      }
      i++;
    }
    
    double ddxx=0.0;
    double ddyy=0.0;
    double ddzz=0.0;
    double dist=0.0;
    double sumdis=0.0;
    
    if(blend){
     
      for(j=0;j<numberV;j++){
	i=0;
	sumdis=0.0;
	path[j].x = 0.0;
	path[j].y = 0.0;
	path[j].z = 0.0;
	now = mycontrol.pHead;
	while(next(now)!=mycontrol.pTail){
	  now = next(now);
	  ddxx = now.gamma*(now.Eweight[j]*(now.Vpoint.x - KnotInit[j].x)/now.norm);
	  ddyy = now.gamma*(now.Eweight[j]*(now.Vpoint.y - KnotInit[j].y)/now.norm);
	  ddzz = now.gamma*(now.Eweight[j]*(now.Vpoint.z - KnotInit[j].z)/now.norm);
	  
	  
	  dist = Math.pow(Math.sqrt((ddxx*ddxx+ddyy*ddyy+ddzz*ddzz)),beta);
	  
	  sumdis += dist;
	  
	  path[j].x -= dist*ddxx;
	  path[j].y -= dist*ddyy;
	  path[j].z -= dist*ddzz;
	  i++;
	}
	if(sumdis!=0.0){
	   path[j].x /= sumdis;
	   path[j].y /= sumdis;
	   path[j].z /= sumdis;
	 }
      }
      
    }else{
       for(j=0;j<numberV;j++){
	i=0;
	sumdis=0.0;
	path[j].x = 0.0;
	path[j].y = 0.0;
	path[j].z = 0.0;
	now = mycontrol.pHead;
	while(next(now)!=mycontrol.pTail){
	  now = next(now);
	  ddxx = now.gamma*(now.Eweight[j]*(now.Vpoint.x - KnotInit[j].x)/now.norm);
	  ddyy = now.gamma*(now.Eweight[j]*(now.Vpoint.y - KnotInit[j].y)/now.norm);
	  ddzz = now.gamma*(now.Eweight[j]*(now.Vpoint.z - KnotInit[j].z)/now.norm);
	  path[j].x -= ddxx;
	  path[j].y -= ddyy;
	  path[j].z -= ddzz;
	  i++;
	}
	
	
      }
    
    }

  }
  
  
    
  
  private void DMeshAlgorithm(){
    int i;
    
    for(i=0;i<numberV;i++){
      point[i].x = KnotInit[i].x +  path[i].x;
      point[i].y = KnotInit[i].y +  path[i].y;
      point[i].z = KnotInit[i].z +  path[i].z;
      
    }  

  }
  
  public void UpdatePolygon(){
    int j=0;
    for(j=0;j<numberF;j++)
      myobj.MakeFace(j,Face[j][0],Face[j][1],Face[j][2],point);
    myobj.MakeArrayDataAFTER();
  }
  
  public int getNearlestVIP(Point3d dv){
    int i=0;
    int tar=0;
    double mindist=0.0;
    double dumdist=0.0;
    mindist = Distance(dv,KnotInit[0]);
    tar = 0;
    for(i=1;i<numberV;i++){
      dumdist = Distance(dv,KnotInit[i]);
      if(dumdist<mindist){
	mindist = dumdist;
	tar = i;
      }
    }
    return tar;
  }
  public int getNearlestV(Point3d dv){
    int i=0;
    int tar=0;
    double mindist=0.0;
    double dumdist=0.0;
    mindist = Distance(dv,point[0]);
    tar = 0;
    for(i=1;i<numberV;i++){
      dumdist = Distance(dv,point[i]);
      if(dumdist<mindist){
	mindist = dumdist;
	tar = i;
      }
    }
    return tar;
  }
  private double B3BF(double t){
    if((-2 < t)&&(t <=-1)){
      double d = (-t-2.0);
       return (-(d*d*d)/6.0);
    }else if((-1 < t)&&(t <=0)){
     return ((-3.0*t*t*t - 6.0*t*t + 4.0)/6.0); 
    }else if((0< t)&&(t <=1)){
      return ((3.0*t*t*t - 6.0*t*t + 4.0)/6.0);
    }else if((1< t)&&(t <2)){
      double d = (t-2.0);
      return (-(d*d*d)/6.0);
    }else{
      return 0.0;
    }
  }
  private double MetaBalls(double t,double a,double b){
    
    if((0<=t)&&(t<(b/3.0))){
      return (a*(1.0-3.0*(t/b)*(t/b)));
    }else if(((b/3.0)<=t)&&(t<=b)){
      return (((3.0*a)/2.0)*(1.0-t/b)*(1.0-t/b));
    }else{
      return 0.0;
    }
  }
  /*
  private double Angle(int di,int dj){
   
    Point3d base = new Point3d(0.0,0.0,0.0);
    Point3d targetfor = new Point3d(0.0,0.0,0.0);
    makeVector(base,mycontrol.IPpoint[di],mycontrol.point[di]);
    makeVector(targetfor,KnotInit[dj],mycontrol.point[di]);
    double dis1 = Point3dSize(base);
    double dis2 = Point3dSize(targetfor);
    if(dis1==0.0||dis2==0.0){
      if(InnerProduct(base,targetfor)>= 0.0){
	return 2.0*Math.PI;
      }else{
	return -2.0*Math.PI;
      }
    }
    double discheck = (InnerProduct(base,targetfor)/(Point3dSize(base)*Point3dSize(targetfor)));
    if(discheck>1.0)discheck = 1.0;
    if(discheck<-1.0)discheck = -1.0;
    
    double angle = Math.acos(discheck);
    return angle;
  }*/

}
